@@ -366,4 +366,7 @@ information read from DATA-FILE (as generated by
 (ert-deftest sqlind-ert-pr54 ()
   (sqlind-ert-check-file-syntax "test-data/pr54.sql" "test-data/pr54-syn.eld"))
 
+(ert-deftest sqlind-ert-pr60 ()
+  (sqlind-ert-check-file-syntax "test-data/pr60.sql" "test-data/pr60-syn.eld"))
+
 ;;; sql-indent-test.el ends here
