@@ -4,9 +4,10 @@
 
 ;; Author: Alex Harsanyi <AlexHarsanyi@gmail.com>
 ;; Created: 27 Sep 2006
-;; Version: 1.1
+;; Version: 1.2
 ;; Keywords: languages sql
 ;; Homepage: https://github.com/alex-hhh/emacs-sql-indent
+;; Package-Requires: ((cl-lib "0.5"))
 ;;
 ;; This program is free software; you can redistribute it and/or modify
 ;; it under the terms of the GNU General Public License as published by
@@ -37,7 +38,6 @@
 
 (require 'sql)
 (require 'cl-lib)
-(eval-when-compile (require 'cc-defs))  ; for c-point
 
 ;;;; General setup
 
@@ -372,7 +372,7 @@ But don't go before LIMIT."
 ;;;;; Find the syntax and beginning of the current block
 
 (defconst sqlind-end-statement-regexp
-  "\\_<end\\_>\\(?:[ \n\r\t]*\\)\\(if\\_>\\|loop\\_>\\|case\\_>\\)?\\(?:[ \n\r\f]*\\)\\([a-z0-9_]+\\)?"
+  "\\_<end\\_>\\(?:[ \t\n\r\t]*\\)\\(if\\_>\\|loop\\_>\\|case\\_>\\)?\\(?:[ \t\n\r\f]*\\)\\([a-z0-9_]+\\)?"
   "Match an end of statement.
 Matches a string like \"end if|loop|case MAYBE-LABEL\".")
 
@@ -422,7 +422,7 @@ See also `sqlind-beginning-of-block'"
 	;; a then keyword only starts a block when it is part of an
 	;; if, case/when or exception statement
 	(cond
-	  ((looking-at "\\(<<[a-z0-9_]+>>\\)?\\(?:[ \n\r\f]*\\)\\(\\(?:els\\)?if\\)\\_>")
+	  ((looking-at "\\(<<[a-z0-9_]+>>\\)?\\(?:[ \t\n\r\f]*\\)\\(\\(?:els\\)?if\\)\\_>")
 	   (let ((if-label (sqlind-match-string 1))
 		 (if-kind (intern (sqlind-match-string 2)))) ; can be if or elsif
 	     (setq if-label (if if-label (substring if-label 2 -2) ""))
@@ -438,7 +438,7 @@ See also `sqlind-beginning-of-block'"
 		       (throw 'finished
 			 (list 'syntax-error
 			       "bad closing for if block" (point) pos))))))))
-	  ((looking-at "\\(<<[a-z0-9_]+>>\\)?\\(?:[ \n\r\f]*\\)case\\_>")
+	  ((looking-at "\\(<<[a-z0-9_]+>>\\)?\\(?:[ \t\n\r\f]*\\)case\\_>")
 	   ;; find the nearest when block, but only if there are no
 	   ;; end statements in the stack
 	   (let ((case-label (sqlind-match-string 1)))
@@ -625,23 +625,63 @@ See also `sqlind-beginning-of-block'"
     (throw 'finished
       (if (null sqlind-end-stmt-stack)
 	  'declare-statement
-	  (list 'syntax-error "nested declare block" (point) (point))))))
+        (list 'syntax-error "nested declare block" (point) (point))))))
+
+(defun sqlind-maybe-skip-mysql-create-options ()
+  "Move point past any MySQL option declarations.
+
+Statements like \"CREATE VIEW\" or \"CREATE TABLE\" can have
+various options betwen the CREATE keyword and the thing being
+created.  If such options exist at (point) the cursor is moved
+past them.
+
+Currently we move over the following options:
+
+  TEMPORARY
+  ALGORITHM = {UNDEFINED | MERGE | TEMPTABLE}
+  DEFINER = { user | CURENT_USER }
+  SQL SECURITY { DEFINER | INVOKER }
+
+We don't consider if the options are valid or not for the thing
+being created.  We just skip any and all of them that are
+present."
+  (when (eq sql-product 'mysql)
+    (catch 'finished
+      (while t
+        (cond
+          ((looking-at "temporary\\_>")
+           (goto-char (match-end 0))
+           (sqlind-forward-syntactic-ws))
+          ((looking-at "\\(definer\\|algorithm\\)\\(\\s-\\|[\n]\\)*=\\(\\s-\\|[\n]\\)*\\S-+")
+           (goto-char (match-end 0))
+           (sqlind-forward-syntactic-ws))
+          ((looking-at "sql\\(\\s-\\|[\n]\\)+security\\(\\s-\\|[\n]\\)+\\S-+")
+           (goto-char (match-end 0))
+           (sqlind-forward-syntactic-ws))
+          (t (throw 'finished nil)))))))
 
 (defun sqlind-maybe-create-statement ()
   "If (point) is on a CREATE statement, report its syntax.
 See also `sqlind-beginning-of-block'"
-  (when (or (looking-at "create\\_>\\(?:[ \n\r\f]+\\)\\(or\\(?:[ \n\r\f]+\\)replace\\_>\\)?")
+  (when (or (looking-at "create\\_>\\(?:[ \t\n\r\f]+\\)\\(or\\(?:[ \t\n\r\f]+\\)replace\\_>\\)?")
             (looking-at "alter\\_>"))
     (prog1 t                            ; make sure we return t
       (save-excursion
 	;; let's see what are we creating
 	(goto-char (match-end 0))
         (sqlind-forward-syntactic-ws)
+        (sqlind-maybe-skip-mysql-create-options)
 	(let ((what (intern (downcase (buffer-substring-no-properties
 				       (point)
 				       (progn (forward-word) (point))))))
 	      (name (downcase (buffer-substring-no-properties
-			       (progn (sqlind-forward-syntactic-ws) (point))
+			       (progn (sqlind-forward-syntactic-ws)
+                                      ;; Skip over a possible "if (not)
+                                      ;; exists", to get the actual name
+                                      (when (looking-at "if\\(\\s-\\|[\n]\\)+\\(not\\)?\\(\\s-\\|[\n]\\)exists")
+                                        (goto-char (match-end 0))
+                                        (sqlind-forward-syntactic-ws))
+                                      (point))
 			       (progn (skip-syntax-forward "w_()") (point))))))
 	  (when (and (eq what 'package) (equal name "body"))
 	    (setq what 'package-body)
@@ -679,7 +719,7 @@ See also `sqlind-beginning-of-block'"
   "If (point) is on a procedure definition statement, report its syntax.
 See also `sqlind-beginning-of-block'"
   (catch 'exit
-    (when (looking-at "\\(procedure\\|function\\)\\(?:[ \n\r\f]+\\)\\([a-z0-9_]+\\)")
+    (when (looking-at "\\(procedure\\|function\\)\\(?:[ \t\n\r\f]+\\)\\([a-z0-9_]+\\)")
       (prog1 t                          ; make sure we return t
 	(let ((proc-name (sqlind-match-string 2)))
 	  ;; need to find out if this is a procedure/function
@@ -691,7 +731,7 @@ See also `sqlind-beginning-of-block'"
 	    (when (looking-at "(")
 	      (ignore-errors (forward-sexp 1))
 	      (sqlind-forward-syntactic-ws))
-	    (when (looking-at "return\\(?:[ \n\r\f]+\\)\\([a-z0-9_.]+\\(?:%\\(?:row\\)?type\\)?\\)")
+	    (when (looking-at "return\\(?:[ \t\n\r\f]+\\)\\([a-z0-9_.]+\\(?:%\\(?:row\\)?type\\)?\\)")
 	      (goto-char (match-end 0))
 	      (sqlind-forward-syntactic-ws))
 	    (when (looking-at ";")
@@ -1391,9 +1431,10 @@ not a statement-continuation POS is the same as the
        (when (sqlind-search-backward pos "when\\_>" anchor)
          (push (cons '(in-block when) (point)) context))))
 
-    ;; indenting the select clause inside a view
+    ;; indenting the select clause inside a view or a "create table as"
+    ;; statement.
     ((and (eq syntax-symbol 'create-statement)
-          (eq (nth 1 syntax) 'view))
+          (memq (nth 1 syntax) '(view table)))
      (goto-char anchor)
      (catch 'done
        (while (re-search-forward "\\bselect\\b" pos 'noerror)
@@ -1890,7 +1931,7 @@ BASE-INDENTATION."
     (if (and (looking-at "\\s *--")
 	     (progn
 	       (forward-line -1)
-	       (re-search-forward "--" (c-point 'eol) t)))
+	       (re-search-forward "--" (line-end-position) t)))
 	(progn
 	  (goto-char (match-beginning 0))
 	  (current-column))
@@ -1912,7 +1953,7 @@ first column after the SELECT clause we simply add
       (goto-char (match-end 0)))
     (skip-syntax-forward " ")
     (if (or (looking-at sqlind-comment-start-skip)
-            (looking-at "$"))
+            (eolp))
         (+ base-indentation sqlind-basic-offset)
       (current-column))))
 
@@ -1930,7 +1971,7 @@ table, we simply add `sqlind-basic-offset'."
       (goto-char (match-end 0)))
     (skip-syntax-forward " ")
     (if (or (looking-at sqlind-comment-start-skip)
-            (looking-at "$"))
+            (eolp))
         (+ base-indentation sqlind-basic-offset)
       (current-column))))
 
@@ -1952,7 +1993,7 @@ Argument BASE-INDENTATION is updated."
       (forward-char (1+ (length clause)))
       (skip-syntax-forward " ")
       (if (or (looking-at sqlind-comment-start-skip)
-              (looking-at "$"))
+              (eolp))
           ;; if the clause is on a line by itself, indent this line with a
           ;; sqlind-basic-offset
           (+ base-indentation sqlind-basic-offset)
